==> apexl-io-3.0.3-34a84a.tar: src/Io/middleware/addCorsHeaders.php <==
<?php

declare(strict_types=1);

namespace apexl\Io\middleware;

use apexl\Io\includes\HookManager;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Slim\Psr7\Factory\ResponseFactory;
use Throwable;

final readonly class addCorsHeaders implements MiddlewareInterface
{
    private ServerRequestInterface $request;

    public function __construct(private HookManager $hookManager) {}

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $this->request = $request;

        try {
            $response = $handler->handle($request);
        } catch (Throwable $throwable) {
            return $this->handleThrowable($throwable);
        }

        return $this->maybeAddCorsHeaders($response);
    }

    private function handleThrowable(Throwable $throwable): ResponseInterface
    {
        error_log(
            sprintf(
                '[IO Exception :: Response] (Line %d | %s) - %s | %s',
                $throwable->getLine(),
                $throwable->getFile(),
                $throwable->getCode(),
                $throwable->getMessage()
            )
        );

        $this->hookManager->processHook('error', $throwable);

        $response = new ResponseFactory()->createResponse();

        return $this->maybeAddCorsHeaders($response)
            ->withHeader('Content-Type', 'application/json')
            ->withStatus(500);
    }

    private function maybeAddCorsHeaders(ResponseInterface $response): ResponseInterface
    {
        $referer = $this->refererFromRequest($this->request);

        if ($referer === null) {
            if (config('app.environment') === 'development') {
                return $this->addCorsHeaders($response);
            }

            return $response;
        }

        if ($domain = $this->matchRefererToAllowedDomains($referer)) {
            return $this->addCorsHeaders($response, $domain);
        }

        return $response;
    }

    private function refererFromRequest(RequestInterface $request): ?string
    {
        return $request->getHeader('HTTP_REFERER')[0] ?? null;
    }

    public function addCorsHeaders(ResponseInterface $response, string $domain = "*"): ResponseInterface
    {
        return $response
            ->withHeader('Access-Control-Allow-Origin', $domain)
            ->withHeader(
                'Access-Control-Allow-Headers',
                implode(', ', [
                    'X-Requested-With',
                    'Content-Type',
                    'Accept',
                    'Origin',
                    'Authorization',
                    ...config('app.allowed_headers'),
                ])
            )
            ->withHeader('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE, PATCH, OPTIONS')
            ->withHeader('Access-Control-Allow-Credentials', $domain === '*' ? 'false' : 'true');
    }

    private function matchRefererToAllowedDomains(string $referer): ?string
    {
        $allowedDomains = config('app.allowed_domains');

        $bits = parse_url($referer);
        $referer = sprintf(
            '%s://%s%s',
            $bits['scheme'],
            $bits['host'],
            ($bits['port'] ?? null) ? sprintf(":%s", $bits['port']) : ''
        );

        $ix = array_search(trim($referer, '/'), $allowedDomains);

        return $ix === false ? null : $allowedDomains[$ix];
    }

}

==> apexl-io-3.0.7-586aab.tar: src/Io/middleware/addCorsHeaders.php <==
<?php

declare(strict_types=1);

namespace apexl\Io\middleware;

use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

final readonly class addCorsHeaders implements MiddlewareInterface
{
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $response = $handler->handle($request);

        return $this->maybeAddCorsHeaders($request, $response);
    }


    private function maybeAddCorsHeaders(ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        $referer = $this->refererFromRequest($request);

        if ($referer === null) {
            if (config('app.environment') === 'development') {
                return $this->addCorsHeaders($response);
            }

            return $response;
        }

        if ($domain = $this->matchRefererToAllowedDomains($referer)) {
            return $this->addCorsHeaders($response, $domain);
        }

        return $response;
    }

    private function refererFromRequest(RequestInterface $request): ?string
    {
        return $request->getHeader('HTTP_REFERER')[0] ?? null;
    }

    public function addCorsHeaders(ResponseInterface $response, string $domain = "*"): ResponseInterface
    {
        return $response
            ->withHeader('Access-Control-Allow-Origin', $domain)
            ->withHeader(
                'Access-Control-Allow-Headers',
                implode(', ', [
                    'X-Requested-With',
                    'Content-Type',
                    'Accept',
                    'Origin',
                    'Authorization',
                    ...config('app.allowed_headers'),
                ])
            )
            ->withHeader('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE, PATCH, OPTIONS')
            ->withHeader('Access-Control-Allow-Credentials', $domain === '*' ? 'false' : 'true');
    }

    private function matchRefererToAllowedDomains(string $referer): ?string
    {
        $allowedDomains = config('app.allowed_domains');

        $bits = parse_url($referer);
        $referer = sprintf(
            '%s://%s%s',
            $bits['scheme'],
            $bits['host'],
            ($bits['port'] ?? null) ? sprintf(":%s", $bits['port']) : ''
        );

        $ix = array_search(trim($referer, '/'), $allowedDomains);

        return $ix === false ? null : $allowedDomains[$ix];
    }

}
